using System;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Interop;

namespace TestProject
{
    /// <summary>
    ///   
    /// </summary>
    public class WindowBlurHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region   Ư ϱ - SetWindowCompositionAttribute(windowHandle, data)

        /// <summary>
        ///   Ư ϱ
        /// </summary>
        /// <param name="windowHandle"> ڵ</param>
        /// <param name="data"></param>
        /// <returns>ó </returns>
        [DllImport("user32.dll")]
        private static extern int SetWindowCompositionAttribute(IntPtr windowHandle, ref WindowCompositionAttributeData data);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// ̿   Ӽ
        /// </summary>
        public static readonly DependencyProperty IsEnabledProperty = DependencyProperty.RegisterAttached
        (
            "IsEnabled",
            typeof(bool),
            typeof(WindowBlurHelper),
            new PropertyMetadata(false, IsEnabledPropertyChangedCallback)
        );

        /// <summary>
        ///    Ӽ
        /// </summary>
        public static readonly DependencyProperty WindowBlurHelperProperty = DependencyProperty.RegisterAttached
        (
            "WindowBlurHelper",
            typeof(WindowBlurHelper),
            typeof(WindowBlurHelper),
            new PropertyMetadata(null, WindowBlurHelperPropertyChangedCallback));

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 
        /// </summary>
        private Window window;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region ̿   ϱ - SetIsEnabled(dependencyObject, value)

        /// <summary>
        /// ̿   ϱ
        /// </summary>
        /// <param name="dependencyObject"> ü</param>
        /// <param name="value"></param>
        public static void SetIsEnabled(DependencyObject dependencyObject, bool value)
        {
            dependencyObject.SetValue(IsEnabledProperty, value);
        }

        #endregion
        #region ̿   ϱ - GetIsEnabled(dependencyObject)

        /// <summary>
        /// ̿   ϱ
        /// </summary>
        /// <param name="dependencyObject"> ü</param>
        /// <returns>̿  </returns>
        public static bool GetIsEnabled(DependencyObject dependencyObject)
        {
            return (bool)dependencyObject.GetValue(IsEnabledProperty);
        }

        #endregion

        #region    ϱ - SetWindowBlurHelper(dependencyObject, value)

        /// <summary>
        ///    ϱ
        /// </summary>
        /// <param name="dependencyObject"> ü</param>
        /// <param name="value"></param>
        public static void SetWindowBlurHelper(DependencyObject dependencyObject, WindowBlurHelper value)
        {
            dependencyObject.SetValue(WindowBlurHelperProperty, value);
        }

        #endregion
        #region    ϱ - GetWindowBlurHelper(dependencyObject)

        /// <summary>
        ///    ϱ
        /// </summary>
        /// <param name="dependencyObject"> ü</param>
        /// <returns>  </returns>
        public static WindowBlurHelper GetWindowBlurHelper(DependencyObject dependencyObject)
        {
            return (WindowBlurHelper)dependencyObject.GetValue(WindowBlurHelperProperty);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region ̿   Ӽ  ݹ óϱ - IsEnabledPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// ̿   Ӽ  ݹ óϱ
        /// </summary>
        /// <param name="dependencyObject"> ü</param>
        /// <param name="e">̺Ʈ </param>
        private static void IsEnabledPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            if(dependencyObject is Window window)
            {
                if(true.Equals(e.OldValue))
                {
                    GetWindowBlurHelper(window)?.Detach();

                    window.ClearValue(WindowBlurHelperProperty);
                }

                if(true.Equals(e.NewValue))
                {
                    WindowBlurHelper helper = new WindowBlurHelper();

                    helper.Attach(window);

                    window.SetValue(WindowBlurHelperProperty, helper);
                }
            }
        }

        #endregion
        #region    Ӽ  ݹ óϱ - WindowBlurHelperPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        ///    Ӽ  ݹ óϱ
        /// </summary>
        /// <param name="dependencyObject"> ü</param>
        /// <param name="e">̺Ʈ </param>
        private static void WindowBlurHelperPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            if(dependencyObject is Window window)
            {
                (e.OldValue as WindowBlurHelper)?.Detach();

                (e.NewValue as WindowBlurHelper)?.Attach(window);
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private
        ////////////////////////////////////////////////////////////////////// Event

        #region  ҽ ʱȭ óϱ - window_SourceInitialized(sender, e)

        /// <summary>
        ///  ҽ ʱȭ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void window_SourceInitialized(object sender, EventArgs e)
        {
            ((Window)sender).SourceInitialized -= window_SourceInitialized;

            AttachCore();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////// Function

        #region  ̿   ϱ - EnableBlur(window)

        /// <summary>
        ///  ̿   ϱ
        /// </summary>
        /// <param name="window"></param>
        private static void EnableBlur(Window window)
        {
            WindowInteropHelper helper = new WindowInteropHelper(window);

            AccentPolicy accentPolicy = new AccentPolicy
            {
                AccentState = AccentState.ACCENT_ENABLE_BLURBEHIND
            };

            int accentPolicySize = Marshal.SizeOf(accentPolicy);

            IntPtr accentPolicyHandle = Marshal.AllocHGlobal(accentPolicySize);

            Marshal.StructureToPtr(accentPolicy, accentPolicyHandle, false);

            WindowCompositionAttributeData data = new WindowCompositionAttributeData
            {
                Attribute = WindowCompositionAttribute.WCA_ACCENT_POLICY,
                DataSize  = accentPolicySize,
                Data      = accentPolicyHandle
            };

            SetWindowCompositionAttribute(helper.Handle, ref data);

            Marshal.FreeHGlobal(accentPolicyHandle);
        }

        #endregion

        #region ÷ϱ (ھ) - AttachCore()

        /// <summary>
        /// ÷ϱ (ھ)
        /// </summary>
        private void AttachCore()
        {
            EnableBlur(this.window);
        }

        #endregion
        #region иϱ (ھ) - DetachCore()

        /// <summary>
        /// иϱ (ھ)
        /// </summary>
        private void DetachCore()
        {
            this.window.SourceInitialized += window_SourceInitialized;
        }

        #endregion

        #region ÷ϱ - Attach(window)

        /// <summary>
        /// ÷ϱ
        /// </summary>
        /// <param name="window"></param>
        private void Attach(Window window)
        {
            this.window = window;

            HwndSource source = PresentationSource.FromVisual(window) as HwndSource;

            if(source == null)
            {
                window.SourceInitialized += window_SourceInitialized;
            }
            else
            {
                AttachCore();
            }
        }

        #endregion
        #region иϱ - Detach()

        /// <summary>
        /// иϱ
        /// </summary>
        private void Detach()
        {
            try
            {
                DetachCore();
            }
            finally
            {
                this.window = null;
            }
        }

        #endregion
    }
}